# -*- coding: UTF-8 -*-
# Author:  Werner F. Bruhin
# Purpose: Application constants
# Created: 06/04/2012
#
# Edited by Gianluca (jeanslack) Pernigotto
# Update: November 29 2020 (update with Russian language)

import wx

# language domain
langDomain = "videomass"
# languages you want to support
supLang = {"en": wx.LANGUAGE_ENGLISH,
           "it": wx.LANGUAGE_ITALIAN,
           "ru": wx.LANGUAGE_RUSSIAN,
           "nl": wx.LANGUAGE_DUTCH,
           "pt": wx.LANGUAGE_PORTUGUESE_BRAZILIAN,
           }
